@@ -1,5 +1,6 @@
 use std::cell::RefCell;
 
+use ntex::io::utils::seal;
 use ntex::service::{fn_factory_with_config, fn_service, ServiceFactory};
 use ntex::util::{ByteString, Ready};
 use ntex_mqtt::v5::{
@@ -30,9 +31,9 @@ impl std::convert::TryFrom<MyServerError> for PublishAck {
     }
 }
 
-async fn handshake<Io>(
-    handshake: v5::Handshake<Io>,
-) -> Result<v5::HandshakeAck<Io, MySession>, MyServerError> {
+async fn handshake(
+    handshake: v5::Handshake,
+) -> Result<v5::HandshakeAck<MySession>, MyServerError> {
     log::info!("new connection: {:?}", handshake);
 
     let session = MySession {
@@ -70,8 +71,8 @@ async fn publish(
 }
 
 fn control_service_factory() -> impl ServiceFactory<
-    Config = Session<MySession>,
-    Request = ControlMessage<MyServerError>,
+    ControlMessage<MyServerError>,
+    Session<MySession>,
     Response = ControlResult,
     Error = MyServerError,
     InitError = MyServerError,
@@ -96,6 +97,7 @@ fn control_service_factory() -> impl ServiceFactory<
             }
             v5::ControlMessage::Unsubscribe(s) => Ready::Ok(s.ack()),
             v5::ControlMessage::Closed(c) => Ready::Ok(c.ack()),
+            v5::ControlMessage::PeerGone(c) => Ready::Ok(c.ack()),
         }))
     })
 }
@@ -106,15 +108,17 @@ async fn main() -> std::io::Result<()> {
     env_logger::init();
 
     ntex::server::Server::build()
-        .bind("mqtt", "127.0.0.1:1883", || {
-            MqttServer::new(handshake)
-                .control(control_service_factory())
-                .publish(fn_factory_with_config(|session: Session<MySession>| {
-                    Ready::Ok::<_, MyServerError>(fn_service(move |req| {
-                        publish(session.clone(), req)
+        .bind("mqtt", "127.0.0.1:1883", |_| {
+            seal(
+                MqttServer::new(handshake)
+                    .control(control_service_factory())
+                    .publish(fn_factory_with_config(|session: Session<MySession>| {
+                        Ready::Ok::<_, MyServerError>(fn_service(move |req| {
+                            publish(session.clone(), req)
+                        }))
                     }))
-                }))
-                .finish()
+                    .finish(),
+            )
         })?
         .workers(1)
         .run()
