@@ -1,5 +1,6 @@
-#![deny(rust_2018_idioms)]
-#![allow(clippy::type_complexity)]
+#![deny(rust_2018_idioms, warnings)]
+#![allow(clippy::type_complexity, clippy::return_self_not_must_use)]
+
 //! MQTT Client/Server framework
 
 #[macro_use]
