@@ -1,78 +1,65 @@
 use std::task::{Context, Poll};
 use std::{fmt, future::Future, marker::PhantomData, pin::Pin, rc::Rc};
 
-use ntex::codec::{AsyncRead, AsyncWrite, Decoder, Encoder};
-use ntex::service::{IntoServiceFactory, Service, ServiceFactory};
-use ntex::time::{Millis, Seconds, Sleep};
-use ntex::util::{select, Either, Pool};
+use ntex::codec::{Decoder, Encoder};
+use ntex::io::{DispatchItem, IoBoxed};
+use ntex::service::{Service, ServiceFactory};
+use ntex::time::{Seconds, Sleep};
+use ntex::util::{select, Either};
 
-use super::io::{DispatchItem, Dispatcher, State, Timer};
+use crate::io::Dispatcher;
 
 type ResponseItem<U> = Option<<U as Encoder>::Item>;
 
-pub(crate) struct FramedService<St, C, T, Io, Codec> {
+pub(crate) struct FramedService<St, C, T, Codec> {
     connect: C,
     handler: Rc<T>,
     disconnect_timeout: Seconds,
-    time: Timer,
-    pool: Pool,
-    _t: PhantomData<(St, Io, Codec)>,
+    _t: PhantomData<(St, Codec)>,
 }
 
-impl<St, C, T, Io, Codec> FramedService<St, C, T, Io, Codec> {
-    pub(crate) fn new(connect: C, service: T, pool: Pool, disconnect_timeout: Seconds) -> Self {
+impl<St, C, T, Codec> FramedService<St, C, T, Codec> {
+    pub(crate) fn new(connect: C, service: T, disconnect_timeout: Seconds) -> Self {
         FramedService {
-            pool,
             connect,
             disconnect_timeout,
             handler: Rc::new(service),
-            time: Timer::new(Millis::ONE_SEC),
             _t: PhantomData,
         }
     }
 }
 
-impl<St, C, T, Io, Codec> ServiceFactory for FramedService<St, C, T, Io, Codec>
+impl<St, C, T, Codec> ServiceFactory<IoBoxed> for FramedService<St, C, T, Codec>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    C: ServiceFactory<Config = (), Request = Io, Response = (Io, State, Codec, St, Seconds)>,
+    St: 'static,
+    C: ServiceFactory<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
     C::Error: fmt::Debug,
-    C::Future: 'static,
-    <C::Service as Service>::Future: 'static,
     T: ServiceFactory<
-            Config = St,
-            Request = DispatchItem<Codec>,
+            DispatchItem<Codec>,
+            St,
             Response = ResponseItem<Codec>,
             Error = C::Error,
             InitError = C::Error,
         > + 'static,
-    <T::Service as Service>::Error: 'static,
-    <T::Service as Service>::Future: 'static,
     Codec: Decoder + Encoder + Clone + 'static,
     <Codec as Encoder>::Item: 'static,
 {
-    type Config = ();
-    type Request = Io;
     type Response = ();
     type Error = C::Error;
     type InitError = C::InitError;
-    type Service = FramedServiceImpl<St, C::Service, T, Io, Codec>;
+    type Service = FramedServiceImpl<St, C::Service, T, Codec>;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
         let fut = self.connect.new_service(());
         let handler = self.handler.clone();
         let disconnect_timeout = self.disconnect_timeout;
-        let time = self.time.clone();
-        let pool = self.pool.clone();
 
         // create connect service and then create service impl
         Box::pin(async move {
             Ok(FramedServiceImpl {
                 handler,
                 disconnect_timeout,
-                pool,
-                time,
                 connect: fut.await?,
                 _t: PhantomData,
             })
@@ -80,48 +67,34 @@ where
     }
 }
 
-pub(crate) struct FramedServiceImpl<St, C, T, Io, Codec> {
+pub(crate) struct FramedServiceImpl<St, C, T, Codec> {
     connect: C,
     handler: Rc<T>,
     disconnect_timeout: Seconds,
-    pool: Pool,
-    time: Timer,
-    _t: PhantomData<(St, Io, Codec)>,
+    _t: PhantomData<(St, Codec)>,
 }
 
-impl<St, C, T, Io, Codec> Service for FramedServiceImpl<St, C, T, Io, Codec>
+impl<St, C, T, Codec> Service<IoBoxed> for FramedServiceImpl<St, C, T, Codec>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    C: Service<Request = Io, Response = (Io, State, Codec, St, Seconds)>,
+    St: 'static,
+    C: Service<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
     C::Error: fmt::Debug,
-    C::Future: 'static,
     T: ServiceFactory<
-            Config = St,
-            Request = DispatchItem<Codec>,
+            DispatchItem<Codec>,
+            St,
             Response = ResponseItem<Codec>,
             Error = C::Error,
             InitError = C::Error,
         > + 'static,
-    <T::Service as Service>::Error: 'static,
-    <T::Service as Service>::Future: 'static,
     Codec: Decoder + Encoder + Clone + 'static,
-    <Codec as Encoder>::Item: 'static,
 {
-    type Request = Io;
     type Response = ();
     type Error = C::Error;
     type Future = Pin<Box<dyn Future<Output = Result<(), Self::Error>>>>;
 
     #[inline]
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
-        let ready1 = self.connect.poll_ready(cx)?.is_ready();
-        let ready2 = self.pool.poll_ready(cx).is_ready();
-
-        if ready1 && ready2 {
-            Poll::Ready(Ok(()))
-        } else {
-            Poll::Pending
-        }
+        self.connect.poll_ready(cx)
     }
 
     #[inline]
@@ -130,16 +103,15 @@ where
     }
 
     #[inline]
-    fn call(&self, req: Io) -> Self::Future {
+    fn call(&self, req: IoBoxed) -> Self::Future {
         log::trace!("Start connection handshake");
 
         let handler = self.handler.clone();
         let timeout = self.disconnect_timeout;
         let handshake = self.connect.call(req);
-        let time = self.time.clone();
 
         Box::pin(async move {
-            let (io, st, codec, session, keepalive) = handshake.await.map_err(|e| {
+            let (io, codec, session, keepalive) = handshake.await.map_err(|e| {
                 log::trace!("Connection handshake failed: {:?}", e);
                 e
             })?;
@@ -148,7 +120,7 @@ where
             let handler = handler.new_service(session).await?;
             log::trace!("Connection handler is created, starting dispatcher");
 
-            Dispatcher::with(io, st, codec, handler, time)
+            Dispatcher::new(io, codec, handler)
                 .keepalive_timeout(keepalive)
                 .disconnect_timeout(timeout)
                 .await
@@ -156,73 +128,55 @@ where
     }
 }
 
-pub(crate) struct FramedService2<St, C, T, Io, Codec> {
+pub(crate) struct FramedService2<St, C, T, Codec> {
     connect: C,
     handler: Rc<T>,
     disconnect_timeout: Seconds,
-    pool: Pool,
-    time: Timer,
-    _t: PhantomData<(St, Io, Codec)>,
+    _t: PhantomData<(St, Codec)>,
 }
 
-impl<St, C, T, Io, Codec> FramedService2<St, C, T, Io, Codec> {
-    pub(crate) fn new(connect: C, service: T, pool: Pool, disconnect_timeout: Seconds) -> Self {
+impl<St, C, T, Codec> FramedService2<St, C, T, Codec> {
+    pub(crate) fn new(connect: C, service: T, disconnect_timeout: Seconds) -> Self {
         FramedService2 {
             connect,
-            pool,
             disconnect_timeout,
             handler: Rc::new(service),
-            time: Timer::new(Millis::ONE_SEC),
             _t: PhantomData,
         }
     }
 }
 
-impl<St, C, T, Io, Codec> ServiceFactory for FramedService2<St, C, T, Io, Codec>
+impl<St, C, T, Codec> ServiceFactory<(IoBoxed, Option<Sleep>)>
+    for FramedService2<St, C, T, Codec>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    C: ServiceFactory<
-        Config = (),
-        Request = (Io, State),
-        Response = (Io, State, Codec, St, Seconds),
-    >,
+    St: 'static,
+    C: ServiceFactory<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
     C::Error: fmt::Debug,
-    C::Future: 'static,
-    <C::Service as Service>::Future: 'static,
     T: ServiceFactory<
-            Config = St,
-            Request = DispatchItem<Codec>,
+            DispatchItem<Codec>,
+            St,
             Response = ResponseItem<Codec>,
             Error = C::Error,
             InitError = C::Error,
         > + 'static,
-    <T::Service as Service>::Error: 'static,
-    <T::Service as Service>::Future: 'static,
     Codec: Decoder + Encoder + Clone + 'static,
-    <Codec as Encoder>::Item: 'static,
 {
-    type Config = ();
-    type Request = (Io, State, Option<Sleep>);
     type Response = ();
     type Error = C::Error;
     type InitError = C::InitError;
-    type Service = FramedServiceImpl2<St, C::Service, T, Io, Codec>;
+    type Service = FramedServiceImpl2<St, C::Service, T, Codec>;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
         let fut = self.connect.new_service(());
         let handler = self.handler.clone();
         let disconnect_timeout = self.disconnect_timeout;
-        let time = self.time.clone();
-        let pool = self.pool.clone();
 
         // create connect service and then create service impl
         Box::pin(async move {
             Ok(FramedServiceImpl2 {
                 handler,
                 disconnect_timeout,
-                time,
-                pool,
                 connect: fut.await?,
                 _t: PhantomData,
             })
@@ -230,48 +184,34 @@ where
     }
 }
 
-pub(crate) struct FramedServiceImpl2<St, C, T, Io, Codec> {
+pub(crate) struct FramedServiceImpl2<St, C, T, Codec> {
     connect: C,
     handler: Rc<T>,
-    pool: Pool,
     disconnect_timeout: Seconds,
-    time: Timer,
-    _t: PhantomData<(St, Io, Codec)>,
+    _t: PhantomData<(St, Codec)>,
 }
 
-impl<St, C, T, Io, Codec> Service for FramedServiceImpl2<St, C, T, Io, Codec>
+impl<St, C, T, Codec> Service<(IoBoxed, Option<Sleep>)> for FramedServiceImpl2<St, C, T, Codec>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    C: Service<Request = (Io, State), Response = (Io, State, Codec, St, Seconds)>,
+    St: 'static,
+    C: Service<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
     C::Error: fmt::Debug,
-    C::Future: 'static,
     T: ServiceFactory<
-            Config = St,
-            Request = DispatchItem<Codec>,
+            DispatchItem<Codec>,
+            St,
             Response = ResponseItem<Codec>,
             Error = C::Error,
             InitError = C::Error,
         > + 'static,
-    <T::Service as Service>::Error: 'static,
-    <T::Service as Service>::Future: 'static,
     Codec: Decoder + Encoder + Clone + 'static,
-    <Codec as Encoder>::Item: 'static,
 {
-    type Request = (Io, State, Option<Sleep>);
     type Response = ();
     type Error = C::Error;
     type Future = Pin<Box<dyn Future<Output = Result<(), Self::Error>>>>;
 
     #[inline]
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
-        let ready1 = self.connect.poll_ready(cx)?.is_ready();
-        let ready2 = self.pool.poll_ready(cx).is_ready();
-
-        if ready1 && ready2 {
-            Poll::Ready(Ok(()))
-        } else {
-            Poll::Pending
-        }
+        self.connect.poll_ready(cx)
     }
 
     #[inline]
@@ -280,20 +220,19 @@ where
     }
 
     #[inline]
-    fn call(&self, (req, state, delay): (Io, State, Option<Sleep>)) -> Self::Future {
+    fn call(&self, (req, delay): (IoBoxed, Option<Sleep>)) -> Self::Future {
         log::trace!("Start connection handshake");
 
         let handler = self.handler.clone();
         let timeout = self.disconnect_timeout;
-        let handshake = self.connect.call((req, state));
-        let time = self.time.clone();
+        let handshake = self.connect.call(req);
 
         Box::pin(async move {
-            let (io, state, codec, ka, handler) = if let Some(delay) = delay {
+            let (io, codec, ka, handler) = if let Some(delay) = delay {
                 let res = select(
                     delay,
                     Box::pin(async {
-                        let (io, state, codec, st, ka) = handshake.await.map_err(|e| {
+                        let (io, codec, st, ka) = handshake.await.map_err(|e| {
                             log::trace!("Connection handshake failed: {:?}", e);
                             e
                         })?;
@@ -302,7 +241,7 @@ where
                         let handler = handler.new_service(st).await?;
                         log::trace!("Connection handler is created, starting dispatcher");
 
-                        Ok::<_, C::Error>((io, state, codec, ka, handler))
+                        Ok::<_, C::Error>((io, codec, ka, handler))
                     }),
                 )
                 .await;
@@ -315,7 +254,7 @@ where
                     Either::Right(item) => item?,
                 }
             } else {
-                let (io, state, codec, st, ka) = handshake.await.map_err(|e| {
+                let (io, codec, st, ka) = handshake.await.map_err(|e| {
                     log::trace!("Connection handshake failed: {:?}", e);
                     e
                 })?;
@@ -323,10 +262,10 @@ where
 
                 let handler = handler.new_service(st).await?;
                 log::trace!("Connection handler is created, starting dispatcher");
-                (io, state, codec, ka, handler)
+                (io, codec, ka, handler)
             };
 
-            Dispatcher::with(io, state, codec, handler, time)
+            Dispatcher::new(io, codec, handler)
                 .keepalive_timeout(ka)
                 .disconnect_timeout(timeout)
                 .await
