@@ -1,13 +1,13 @@
-use std::{fmt, future::Future, marker::PhantomData, rc::Rc, time::Instant};
+use std::{fmt, future::Future, marker::PhantomData, rc::Rc};
 
-use ntex::codec::{AsyncRead, AsyncWrite};
+use ntex::io::IoBoxed;
 use ntex::router::{IntoPattern, Router, RouterBuilder};
-use ntex::service::{apply_fn, boxed, into_service, IntoService, Service};
+use ntex::service::{boxed, into_service, IntoService, Service};
 use ntex::time::{sleep, Millis, Seconds};
 use ntex::util::{Either, Ready};
 
-use crate::error::{MqttError, ProtocolError};
-use crate::io::{DispatchItem, Dispatcher, Timer};
+use crate::error::MqttError;
+use crate::io::Dispatcher;
 use crate::v3::{shared::MqttShared, sink::MqttSink};
 use crate::v3::{ControlResult, Publish};
 
@@ -15,8 +15,8 @@ use super::control::ControlMessage;
 use super::dispatcher::create_dispatcher;
 
 /// Mqtt client
-pub struct Client<Io> {
-    io: Io,
+pub struct Client {
+    io: IoBoxed,
     shared: Rc<MqttShared>,
     keepalive: Seconds,
     disconnect_timeout: Seconds,
@@ -24,7 +24,7 @@ pub struct Client<Io> {
     max_receive: usize,
 }
 
-impl<Io> fmt::Debug for Client<Io> {
+impl fmt::Debug for Client {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("v3::Client")
             .field("keepalive", &self.keepalive)
@@ -35,13 +35,10 @@ impl<Io> fmt::Debug for Client<Io> {
     }
 }
 
-impl<T> Client<T>
-where
-    T: AsyncRead + AsyncWrite + Unpin,
-{
+impl Client {
     /// Construct new `Dispatcher` instance with outgoing messages stream.
     pub(super) fn new(
-        io: T,
+        io: IoBoxed,
         shared: Rc<MqttShared>,
         session_present: bool,
         keepalive_timeout: Seconds,
@@ -59,10 +56,7 @@ where
     }
 }
 
-impl<Io> Client<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+impl Client {
     #[inline]
     /// Get client sink
     pub fn sink(&self) -> MqttSink {
@@ -76,11 +70,11 @@ where
     }
 
     /// Configure mqtt resource for a specific topic
-    pub fn resource<T, F, U, E>(self, address: T, service: F) -> ClientRouter<Io, E, U::Error>
+    pub fn resource<T, F, U, E>(self, address: T, service: F) -> ClientRouter<E, U::Error>
     where
         T: IntoPattern,
-        F: IntoService<U>,
-        U: Service<Request = Publish, Response = ()> + 'static,
+        F: IntoService<U, Publish>,
+        U: Service<Publish, Response = ()> + 'static,
         E: From<U::Error>,
     {
         let mut builder = Router::build();
@@ -114,24 +108,18 @@ where
             into_service(|msg: ControlMessage<()>| Ready::<_, ()>::Ok(msg.disconnect())),
         );
 
-        let _ = Dispatcher::with(
-            self.io,
-            self.shared.state.clone(),
-            self.shared.clone(),
-            dispatcher,
-            Timer::new(Millis::ONE_SEC),
-        )
-        .keepalive_timeout(Seconds::ZERO)
-        .disconnect_timeout(self.disconnect_timeout)
-        .await;
+        let _ = Dispatcher::new(self.io, self.shared.clone(), dispatcher)
+            .keepalive_timeout(Seconds::ZERO)
+            .disconnect_timeout(self.disconnect_timeout)
+            .await;
     }
 
     /// Run client with provided control messages handler
     pub async fn start<F, S, E>(self, service: F) -> Result<(), MqttError<E>>
     where
         E: 'static,
-        F: IntoService<S> + 'static,
-        S: Service<Request = ControlMessage<E>, Response = ControlResult, Error = E> + 'static,
+        F: IntoService<S, ControlMessage<E>> + 'static,
+        S: Service<ControlMessage<E>, Response = ControlResult, Error = E> + 'static,
     {
         if self.keepalive.non_zero() {
             ntex::rt::spawn(keepalive(MqttSink::new(self.shared.clone()), self.keepalive));
@@ -144,26 +132,20 @@ where
             service.into_service(),
         );
 
-        Dispatcher::with(
-            self.io,
-            self.shared.state.clone(),
-            self.shared.clone(),
-            dispatcher,
-            Timer::new(Millis::ONE_SEC),
-        )
-        .keepalive_timeout(Seconds::ZERO)
-        .disconnect_timeout(self.disconnect_timeout)
-        .await
+        Dispatcher::new(self.io, self.shared.clone(), dispatcher)
+            .keepalive_timeout(Seconds::ZERO)
+            .disconnect_timeout(self.disconnect_timeout)
+            .await
     }
 }
 
 type Handler<E> = boxed::BoxService<Publish, (), E>;
 
 /// Mqtt client with routing capabilities
-pub struct ClientRouter<Io, Err, PErr> {
+pub struct ClientRouter<Err, PErr> {
     builder: RouterBuilder<usize>,
     handlers: Vec<Handler<PErr>>,
-    io: Io,
+    io: IoBoxed,
     shared: Rc<MqttShared>,
     keepalive: Seconds,
     disconnect_timeout: Seconds,
@@ -171,7 +153,7 @@ pub struct ClientRouter<Io, Err, PErr> {
     _t: PhantomData<Err>,
 }
 
-impl<Io, Err, PErr> fmt::Debug for ClientRouter<Io, Err, PErr> {
+impl<Err, PErr> fmt::Debug for ClientRouter<Err, PErr> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("v3::ClientRouter")
             .field("keepalive", &self.keepalive)
@@ -181,9 +163,8 @@ impl<Io, Err, PErr> fmt::Debug for ClientRouter<Io, Err, PErr> {
     }
 }
 
-impl<Io, Err, PErr> ClientRouter<Io, Err, PErr>
+impl<Err, PErr> ClientRouter<Err, PErr>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: From<PErr> + 'static,
     PErr: 'static,
 {
@@ -191,8 +172,8 @@ where
     pub fn resource<T, F, S>(mut self, address: T, service: F) -> Self
     where
         T: IntoPattern,
-        F: IntoService<S>,
-        S: Service<Request = Publish, Response = (), Error = PErr> + 'static,
+        F: IntoService<S, Publish>,
+        S: Service<Publish, Response = (), Error = PErr> + 'static,
     {
         self.builder.path(address, self.handlers.len());
         self.handlers.push(boxed::service(service.into_service()));
@@ -212,24 +193,17 @@ where
             into_service(|msg: ControlMessage<Err>| Ready::<_, Err>::Ok(msg.disconnect())),
         );
 
-        let _ = Dispatcher::with(
-            self.io,
-            self.shared.state.clone(),
-            self.shared.clone(),
-            dispatcher,
-            Timer::new(Millis::ONE_SEC),
-        )
-        .keepalive_timeout(Seconds::ZERO)
-        .disconnect_timeout(self.disconnect_timeout)
-        .await;
+        let _ = Dispatcher::new(self.io, self.shared.clone(), dispatcher)
+            .keepalive_timeout(Seconds::ZERO)
+            .disconnect_timeout(self.disconnect_timeout)
+            .await;
     }
 
     /// Run client and handle control messages
     pub async fn start<F, S>(self, service: F) -> Result<(), MqttError<Err>>
     where
-        F: IntoService<S> + 'static,
-        S: Service<Request = ControlMessage<Err>, Response = ControlResult, Error = Err>
-            + 'static,
+        F: IntoService<S, ControlMessage<Err>>,
+        S: Service<ControlMessage<Err>, Response = ControlResult, Error = Err> + 'static,
     {
         if self.keepalive.non_zero() {
             ntex::rt::spawn(keepalive(MqttSink::new(self.shared.clone()), self.keepalive));
@@ -242,23 +216,17 @@ where
             service.into_service(),
         );
 
-        Dispatcher::with(
-            self.io,
-            self.shared.state.clone(),
-            self.shared.clone(),
-            dispatcher,
-            Timer::new(Millis::ONE_SEC),
-        )
-        .keepalive_timeout(Seconds::ZERO)
-        .disconnect_timeout(self.disconnect_timeout)
-        .await
+        Dispatcher::new(self.io, self.shared.clone(), dispatcher)
+            .keepalive_timeout(Seconds::ZERO)
+            .disconnect_timeout(self.disconnect_timeout)
+            .await
     }
 }
 
 fn dispatch<Err, PErr>(
     router: Router<usize>,
     handlers: Vec<Handler<PErr>>,
-) -> impl Service<Request = Publish, Response = Either<(), Publish>, Error = Err>
+) -> impl Service<Publish, Response = Either<(), Publish>, Error = Err>
 where
     PErr: 'static,
     Err: From<PErr>,
@@ -279,7 +247,7 @@ fn call<S, Err, PErr>(
     srv: &S,
 ) -> impl Future<Output = Result<Either<(), Publish>, Err>>
 where
-    S: Service<Request = Publish, Response = (), Error = PErr>,
+    S: Service<Publish, Response = (), Error = PErr>,
     Err: From<PErr>,
 {
     let fut = srv.call(req);
