@@ -1,10 +1,10 @@
 use std::{future::Future, rc::Rc};
 
-use ntex::codec::{AsyncRead, AsyncWrite};
 use ntex::connect::{self, Address, Connect, Connector};
+use ntex::io::{utils::Boxed, Filter, Io, IoBoxed};
 use ntex::service::Service;
-use ntex::time::{timeout, Millis, Seconds};
-use ntex::util::{select, ByteString, Bytes, Either, PoolId};
+use ntex::time::{timeout, Seconds};
+use ntex::util::{ByteString, Bytes, Either, PoolId};
 
 #[cfg(feature = "openssl")]
 use ntex::connect::openssl::{OpensslConnector, SslConnector};
@@ -13,7 +13,6 @@ use ntex::connect::openssl::{OpensslConnector, SslConnector};
 use ntex::connect::rustls::{ClientConfig, RustlsConnector};
 
 use super::{codec, connection::Client, error::ClientError, error::ProtocolError};
-use crate::io::State;
 use crate::v3::shared::{MqttShared, MqttSinkPool};
 
 /// Mqtt client connector
@@ -35,11 +34,11 @@ where
 {
     #[allow(clippy::new_ret_no_self)]
     /// Create new mqtt connector
-    pub fn new(address: A) -> MqttConnector<A, Connector<A>> {
+    pub fn new(address: A) -> MqttConnector<A, Boxed<Connector<A>, Connect<A>>> {
         MqttConnector {
             address,
             pkt: codec::Connect::default(),
-            connector: Connector::default(),
+            connector: Boxed::new(Connector::default()),
             max_send: 16,
             max_receive: 16,
             max_packet_size: 64 * 1024,
@@ -53,8 +52,7 @@ where
 impl<A, T> MqttConnector<A, T>
 where
     A: Address + Clone,
-    T: Service<Request = Connect<A>, Error = connect::ConnectError>,
-    T::Response: AsyncRead + AsyncWrite + Unpin + 'static,
+    T: Service<Connect<A>, Response = IoBoxed, Error = connect::ConnectError>,
 {
     #[inline]
     /// Create new client and provide client id
@@ -179,13 +177,13 @@ where
     }
 
     /// Use custom connector
-    pub fn connector<U>(self, connector: U) -> MqttConnector<A, U>
+    pub fn connector<U, F>(self, connector: U) -> MqttConnector<A, Boxed<U, Connect<A>>>
     where
-        U: Service<Request = Connect<A>, Error = connect::ConnectError>,
-        U::Response: AsyncRead + AsyncWrite + Unpin + 'static,
+        F: Filter,
+        U: Service<Connect<A>, Response = Io<F>, Error = connect::ConnectError>,
     {
         MqttConnector {
-            connector,
+            connector: Boxed::new(connector),
             pkt: self.pkt,
             address: self.address,
             max_send: self.max_send,
@@ -199,14 +197,20 @@ where
 
     #[cfg(feature = "openssl")]
     /// Use openssl connector
-    pub fn openssl(self, connector: SslConnector) -> MqttConnector<A, OpensslConnector<A>> {
+    pub fn openssl(
+        self,
+        connector: SslConnector,
+    ) -> MqttConnector<
+        A,
+        impl Service<Connect<A>, Response = IoBoxed, Error = connect::ConnectError>,
+    > {
         MqttConnector {
             pkt: self.pkt,
             address: self.address,
             max_send: self.max_send,
             max_receive: self.max_receive,
             max_packet_size: self.max_packet_size,
-            connector: OpensslConnector::new(connector),
+            connector: OpensslConnector::new(connector).map(|io| io.into_boxed()),
             handshake_timeout: self.handshake_timeout,
             disconnect_timeout: self.disconnect_timeout,
             pool: self.pool,
@@ -215,16 +219,20 @@ where
 
     #[cfg(feature = "rustls")]
     /// Use rustls connector
-    pub fn rustls(self, config: ClientConfig) -> MqttConnector<A, RustlsConnector<A>> {
-        use std::sync::Arc;
-
+    pub fn rustls(
+        self,
+        config: ClientConfig,
+    ) -> MqttConnector<
+        A,
+        impl Service<Connect<A>, Response = IoBoxed, Error = connect::ConnectError>,
+    > {
         MqttConnector {
             pkt: self.pkt,
             address: self.address,
             max_send: self.max_send,
             max_receive: self.max_receive,
             max_packet_size: self.max_packet_size,
-            connector: RustlsConnector::new(Arc::new(config)),
+            connector: RustlsConnector::new(Arc::new(config)).map(|io| io.into_boxed()),
             handshake_timeout: self.handshake_timeout,
             disconnect_timeout: self.disconnect_timeout,
             pool: self.pool,
@@ -232,7 +240,7 @@ where
     }
 
     /// Connect to mqtt server
-    pub fn connect(&self) -> impl Future<Output = Result<Client<T::Response>, ClientError>> {
+    pub fn connect(&self) -> impl Future<Output = Result<Client, ClientError>> {
         if self.handshake_timeout.non_zero() {
             let fut = timeout(self.handshake_timeout, self._connect());
             Either::Left(async move {
@@ -246,7 +254,7 @@ where
         }
     }
 
-    fn _connect(&self) -> impl Future<Output = Result<Client<T::Response>, ClientError>> {
+    fn _connect(&self) -> impl Future<Output = Result<Client, ClientError>> {
         let fut = self.connector.call(Connect::new(self.address.clone()));
         let pkt = self.pkt.clone();
         let max_send = self.max_send;
@@ -257,23 +265,18 @@ where
         let pool = self.pool.clone();
 
         async move {
-            let mut io = fut.await?;
-            let state = State::with_memory_pool(pool.pool.get());
+            let io = fut.await?;
             let codec = codec::Codec::new().max_size(max_packet_size);
 
-            state.send(&mut io, &codec, pkt.into()).await?;
+            io.send(pkt.into(), &codec).await?;
 
-            let packet = state
-                .next(&mut io, &codec)
-                .await
-                .map_err(|e| ClientError::from(ProtocolError::from(e)))
-                .and_then(|res| {
-                    res.ok_or_else(|| {
-                        log::trace!("Mqtt server is disconnected during handshake");
-                        ClientError::Disconnected
-                    })
+            let packet =
+                io.recv(&codec).await.map_err(ClientError::from)?.ok_or_else(|| {
+                    log::trace!("Mqtt server is disconnected during handshake");
+                    ClientError::Disconnected(None)
                 })?;
-            let shared = Rc::new(MqttShared::new(state.clone(), codec, max_send, pool));
+
+            let shared = Rc::new(MqttShared::new(io.get_ref(), codec, max_send, pool));
 
             match packet {
                 codec::Packet::ConnectAck { session_present, return_code } => {
