@@ -1,9 +1,10 @@
+//! MQTT 3.1.1 client
 mod connection;
 mod connector;
 pub mod control;
 mod dispatcher;
 
-pub use self::connection::Client;
+pub use self::connection::{Client, ClientRouter};
 pub use self::connector::MqttConnector;
 pub use self::control::{ControlMessage, ControlResult};
 
