@@ -1,49 +1,39 @@
 use std::{
     convert::TryFrom, fmt, future::Future, marker, pin::Pin, rc::Rc, task::Context, task::Poll,
-    time,
 };
 
-use ntex::codec::{AsyncRead, AsyncWrite};
-use ntex::service::{apply_fn_factory, boxed, IntoServiceFactory, Service, ServiceFactory};
+use ntex::service::{boxed, Service, ServiceFactory};
 use ntex::time::{sleep, Seconds, Sleep};
-use ntex::util::{timeout::Timeout, timeout::TimeoutError, Either, PoolId, Ready};
+use ntex::{io::IoBoxed, util::Either};
 
 use crate::error::{MqttError, ProtocolError};
-use crate::io::{DispatchItem, State};
 
 use super::control::{ControlMessage, ControlResult};
-use super::default::{DefaultControlService, DefaultPublishService};
 use super::handshake::{Handshake, HandshakeAck};
 use super::publish::{Publish, PublishAck};
 use super::shared::{MqttShared, MqttSinkPool};
-use super::{codec as mqtt, dispatcher::factory, MqttServer, MqttSink, Session};
+use super::{codec as mqtt, MqttServer, Session};
 
-pub(crate) type SelectItem<Io> = (Handshake<Io>, State, Option<Sleep>);
+pub(crate) type SelectItem = (Handshake, Option<Sleep>);
 
-type ServerFactory<Io, Err, InitErr> = boxed::BoxServiceFactory<
-    (),
-    SelectItem<Io>,
-    Either<SelectItem<Io>, ()>,
-    MqttError<Err>,
-    InitErr,
->;
+type ServerFactory<Err, InitErr> =
+    boxed::BoxServiceFactory<(), SelectItem, Either<SelectItem, ()>, MqttError<Err>, InitErr>;
 
-type Server<Io, Err> =
-    boxed::BoxService<SelectItem<Io>, Either<SelectItem<Io>, ()>, MqttError<Err>>;
+type Server<Err> = boxed::BoxService<SelectItem, Either<SelectItem, ()>, MqttError<Err>>;
 
 /// Mqtt server selector
 ///
 /// Selector allows to choose different mqtt server impls depends on
 /// connectt packet.
-pub struct Selector<Io, Err, InitErr> {
-    servers: Vec<ServerFactory<Io, Err, InitErr>>,
+pub struct Selector<Err, InitErr> {
+    servers: Vec<ServerFactory<Err, InitErr>>,
     max_size: u32,
     handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
-    _t: marker::PhantomData<(Io, Err, InitErr)>,
+    _t: marker::PhantomData<(Err, InitErr)>,
 }
 
-impl<Io, Err, InitErr> Selector<Io, Err, InitErr> {
+impl<Err, InitErr> Selector<Err, InitErr> {
     #[allow(clippy::new_without_default)]
     pub fn new() -> Self {
         Selector {
@@ -56,9 +46,8 @@ impl<Io, Err, InitErr> Selector<Io, Err, InitErr> {
     }
 }
 
-impl<Io, Err, InitErr> Selector<Io, Err, InitErr>
+impl<Err, InitErr> Selector<Err, InitErr>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
     InitErr: 'static,
 {
@@ -80,29 +69,19 @@ where
         self
     }
 
-    /// Set memory pool.
-    ///
-    /// Use specified memory pool for memory allocations. By default P5
-    /// memory pool is used.
-    pub fn memory_pool(self, id: PoolId) -> Self {
-        self.pool.pool.set(id.pool_ref());
-        self
-    }
-
     /// Add server variant
     pub fn variant<F, R, St, C, Cn, P>(
         mut self,
         check: F,
-        mut server: MqttServer<Io, St, C, Cn, P>,
+        mut server: MqttServer<St, C, Cn, P>,
     ) -> Self
     where
-        F: Fn(&Handshake<Io>) -> R + 'static,
+        F: Fn(&Handshake) -> R + 'static,
         R: Future<Output = Result<bool, Err>> + 'static,
         St: 'static,
         C: ServiceFactory<
-                Config = (),
-                Request = Handshake<Io>,
-                Response = HandshakeAck<Io, St>,
+                Handshake,
+                Response = HandshakeAck<St>,
                 Error = Err,
                 InitError = InitErr,
             > + 'static,
@@ -111,14 +90,10 @@ where
             + From<P::Error>
             + From<P::InitError>
             + fmt::Debug,
-        Cn: ServiceFactory<
-                Config = Session<St>,
-                Request = ControlMessage<C::Error>,
-                Response = ControlResult,
-            > + 'static,
-
-        P: ServiceFactory<Config = Session<St>, Request = Publish, Response = PublishAck>
+        Cn: ServiceFactory<ControlMessage<Err>, Session<St>, Response = ControlResult>
             + 'static,
+
+        P: ServiceFactory<Publish, Session<St>, Response = PublishAck> + 'static,
         P::Error: fmt::Debug,
         PublishAck: TryFrom<P::Error, Error = C::Error>,
     {
@@ -131,8 +106,7 @@ where
     pub(crate) fn finish_server(
         self,
     ) -> impl ServiceFactory<
-        Config = (),
-        Request = (Io, State, Option<Sleep>),
+        (IoBoxed, Option<Sleep>),
         Response = (),
         Error = MqttError<Err>,
         InitError = InitErr,
@@ -146,18 +120,15 @@ where
     }
 }
 
-impl<Io, Err, InitErr> ServiceFactory for Selector<Io, Err, InitErr>
+impl<Err, InitErr> ServiceFactory<IoBoxed> for Selector<Err, InitErr>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
     InitErr: 'static,
 {
-    type Config = ();
-    type Request = Io;
     type Response = ();
     type Error = MqttError<Err>;
     type InitError = InitErr;
-    type Service = SelectorService<Io, Err>;
+    type Service = SelectorService<Err>;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
@@ -176,19 +147,17 @@ where
     }
 }
 
-pub struct SelectorService<Io, Err> {
-    servers: Rc<Vec<Server<Io, Err>>>,
+pub struct SelectorService<Err> {
+    servers: Rc<Vec<Server<Err>>>,
     max_size: u32,
     handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
 }
 
-impl<Io, Err> Service for SelectorService<Io, Err>
+impl<Err> Service<IoBoxed> for SelectorService<Err>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
 {
-    type Request = Io;
     type Response = ();
     type Error = MqttError<Err>;
     type Future = Pin<Box<dyn Future<Output = Result<(), MqttError<Err>>>>>;
@@ -220,11 +189,10 @@ where
     }
 
     #[inline]
-    fn call(&self, mut io: Io) -> Self::Future {
+    fn call(&self, io: IoBoxed) -> Self::Future {
         let servers = self.servers.clone();
-        let state = State::with_memory_pool(self.pool.pool.get());
         let shared = Rc::new(MqttShared::new(
-            state.clone(),
+            io.get_ref(),
             mqtt::Codec::default().max_inbound_size(self.max_size),
             0,
             self.pool.clone(),
@@ -233,18 +201,16 @@ where
         let delay = self.handshake_timeout.map(sleep);
         Box::pin(async move {
             // read first packet
-            let packet = state
-                .next(&mut io, &shared.codec)
+            let packet = io
+                .recv(&shared.codec)
                 .await
                 .map_err(|err| {
                     log::trace!("Error is received during mqtt handshake: {:?}", err);
                     MqttError::from(err)
-                })
-                .and_then(|res| {
-                    res.ok_or_else(|| {
-                        log::trace!("Server mqtt is disconnected during handshake");
-                        MqttError::Disconnected
-                    })
+                })?
+                .ok_or_else(|| {
+                    log::trace!("Server mqtt is disconnected during handshake");
+                    MqttError::Disconnected(None)
                 })?;
 
             let connect = match packet {
@@ -259,7 +225,7 @@ where
             };
 
             // call servers
-            let mut item = (Handshake::new(connect, io, shared, 0, 0, 0), state, delay);
+            let mut item = (Handshake::new(connect, io, shared, 0, 0, 0), delay);
             for srv in servers.iter() {
                 match srv.call(item).await? {
                     Either::Left(result) => {
@@ -274,25 +240,22 @@ where
     }
 }
 
-pub(crate) struct Selector2<Io, Err, InitErr> {
-    servers: Vec<ServerFactory<Io, Err, InitErr>>,
+pub(crate) struct Selector2<Err, InitErr> {
+    servers: Vec<ServerFactory<Err, InitErr>>,
     max_size: u32,
     pool: Rc<MqttSinkPool>,
-    _t: marker::PhantomData<(Io, Err, InitErr)>,
+    _t: marker::PhantomData<(Err, InitErr)>,
 }
 
-impl<Io, Err, InitErr> ServiceFactory for Selector2<Io, Err, InitErr>
+impl<Err, InitErr> ServiceFactory<(IoBoxed, Option<Sleep>)> for Selector2<Err, InitErr>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
     InitErr: 'static,
 {
-    type Config = ();
-    type Request = (Io, State, Option<Sleep>);
     type Response = ();
     type Error = MqttError<Err>;
     type InitError = InitErr;
-    type Service = SelectorService2<Io, Err>;
+    type Service = SelectorService2<Err>;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
@@ -310,18 +273,16 @@ where
     }
 }
 
-pub(crate) struct SelectorService2<Io, Err> {
-    servers: Rc<Vec<Server<Io, Err>>>,
+pub(crate) struct SelectorService2<Err> {
+    servers: Rc<Vec<Server<Err>>>,
     max_size: u32,
     pool: Rc<MqttSinkPool>,
 }
 
-impl<Io, Err> Service for SelectorService2<Io, Err>
+impl<Err> Service<(IoBoxed, Option<Sleep>)> for SelectorService2<Err>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
 {
-    type Request = (Io, State, Option<Sleep>);
     type Response = ();
     type Error = MqttError<Err>;
     type Future = Pin<Box<dyn Future<Output = Result<(), MqttError<Err>>>>>;
@@ -353,10 +314,10 @@ where
     }
 
     #[inline]
-    fn call(&self, (mut io, state, delay): Self::Request) -> Self::Future {
+    fn call(&self, (io, delay): (IoBoxed, Option<Sleep>)) -> Self::Future {
         let servers = self.servers.clone();
         let shared = Rc::new(MqttShared::new(
-            state.clone(),
+            io.get_ref(),
             mqtt::Codec::default().max_inbound_size(self.max_size),
             0,
             self.pool.clone(),
@@ -364,18 +325,16 @@ where
 
         Box::pin(async move {
             // read first packet
-            let packet = state
-                .next(&mut io, &shared.codec)
+            let packet = io
+                .recv(&shared.codec)
                 .await
                 .map_err(|err| {
-                    log::trace!("Error is received during mqtt handshake: {:?}", err);
+                    // log::trace!("Error is received during mqtt handshake: {:?}", err);
                     MqttError::from(err)
-                })
-                .and_then(|res| {
-                    res.ok_or_else(|| {
-                        log::trace!("Server mqtt is disconnected during handshake");
-                        MqttError::Disconnected
-                    })
+                })?
+                .ok_or_else(|| {
+                    log::trace!("Server mqtt is disconnected during handshake");
+                    MqttError::Disconnected(None)
                 })?;
 
             let connect = match packet {
@@ -390,7 +349,7 @@ where
             };
 
             // call servers
-            let mut item = (Handshake::new(connect, io, shared, 0, 0, 0), state, delay);
+            let mut item = (Handshake::new(connect, io, shared, 0, 0, 0), delay);
             for srv in servers.iter() {
                 match srv.call(item).await? {
                     Either::Left(result) => {
